@@ -15,6 +15,193 @@ my %NS = (
   'dc' => 'http://purl.org/dc/elements/1.1/',
 );
 
+my $XC = XML::LibXML::XPathContext->new();
+
+my %GenreTranslate = (
+  'accounting'=>'Бухучет, налогообложение, аудит',
+  'adventure'=>'Приключения',
+  'adv_animal'=>'Природа и животные',
+  'adv_geo'=>'Книги о Путешествиях',
+  'adv_history'=>'Исторические приключения',
+  'adv_maritime'=>'Морские приключения',
+  'adv_western'=>'Вестерны',
+  'antique'=>'Старинная литература',
+  'antique_ant'=>'Античная литература',
+  'antique_east'=>'Древневосточная литература',
+  'antique_european'=>'Европейская старинная литература',
+  'antique_myths'=>'Мифы. Легенды. Эпос',
+  'antique_russian'=>'Древнерусская литература',
+  'aphorism_quote'=>'Афоризмы и цитаты',
+  'architecture_book'=>'Архитектура',
+  'auto_regulations'=>'Автомобили и ПДД',
+  'banking'=>'Банковское дело',
+  'beginning_authors'=>'Начинающие авторы',
+  'samizdat'=>'Самиздат',
+  'children'=>'Книги для детей',
+  'child_adv'=>'Детские приключения',
+  'child_det'=>'Детские детективы',
+  'child_education'=>'Учебная литература',
+  'child_prose'=>'Детская проза',
+  'child_sf'=>'Детская фантастика',
+  'child_tale'=>'Сказки',
+  'child_verse'=>'Детские стихи',
+  'cinema_theatre'=>'Кинематограф, театр',
+  'city_fantasy'=>'Городское фэнтези',
+  'computers'=>'Компьютеры',
+  'comp_db'=>'Базы данных',
+  'comp_hard'=>'Компьютерное Железо',
+  'comp_osnet'=>'ОС и Сети',
+  'comp_programming'=>'Программирование',
+  'comp_soft'=>'Программы',
+  'comp_www'=>'Интернет',
+  'detective'=>'Современные детективы',
+  'det_action'=>'Боевики',
+  'det_classic'=>'Классические детективы',
+  'det_crime'=>'Криминальные боевики',
+  'det_espionage'=>'Шпионские детективы',
+  'det_hard'=>'Крутой детектив',
+  'det_history'=>'Исторические детективы',
+  'det_irony'=>'Иронические детективы',
+  'det_police'=>'Полицейские детективы',
+  'det_political'=>'Политические детективы',
+  'dragon_fantasy'=>'Фэнтези про драконов',
+  'dramaturgy'=>'Драматургия',
+  'economics'=>'Экономика',
+  'essays'=>'Эссе',
+  'fantasy_fight'=>'Боевое фэнтези',
+  'foreign_action'=>'Зарубежные боевики',
+  'foreign_adventure'=>'Зарубежные приключения',
+  'foreign_antique'=>'Зарубежная старинная литература',
+  'foreign_business'=>'Зарубежная деловая литература',
+  'foreign_children'=>'Зарубежные детские книги',
+  'foreign_comp'=>'Зарубежная компьютерная литература',
+  'foreign_contemporary'=>'Современная зарубежная литература',
+  'foreign_desc'=>'Зарубежная справочная литература',
+  'foreign_detective'=>'Зарубежные детективы',
+  'foreign_dramaturgy'=>'Зарубежная драматургия',
+  'foreign_edu'=>'Зарубежная образовательная литература',
+  'foreign_fantasy'=>'Зарубежное фэнтези',
+  'foreign_home'=>'Зарубежная прикладная и научно-популярная литература',
+  'foreign_humor'=>'Зарубежный юмор',
+  'foreign_language'=>'Иностранные языки',
+  'foreign_love'=>'Зарубежные любовные романы',
+  'foreign_other'=>'Зарубежное',
+  'foreign_poetry'=>'Зарубежные стихи',
+  'foreign_prose'=>'Зарубежная классика',
+  'foreign_psychology'=>'Зарубежная психология',
+  'foreign_publicism'=>'Зарубежная публицистика',
+  'foreign_religion'=>'Зарубежная эзотерическая и религиозная литература',
+  'foreign_sf'=>'Зарубежная фантастика',
+  'geography_book'=>'География',
+  'geo_guides'=>'Путеводители',
+  'global_economy'=>'ВЭД',
+  'historical_fantasy'=>'Историческое фэнтези',
+  'home'=>'Дом и Семья',
+  'home_cooking'=>'Кулинария',
+  'home_crafts'=>'Хобби, Ремесла',
+  'home_diy'=>'Сделай Сам',
+  'home_entertain'=>'Развлечения',
+  'home_garden'=>'Сад и Огород',
+  'home_health'=>'Здоровье',
+  'home_pets'=>'Домашние Животные',
+  'home_sex'=>'Эротика, Секс',
+  'home_sport'=>'Спорт, фитнес',
+  'humor'=>'Юмор',
+  'humor_anecdote'=>'Анекдоты',
+  'humor_fantasy'=>'Юмористическое фэнтези',
+  'humor_prose'=>'Юмористическая проза',
+  'humor_verse'=>'Юмористические стихи',
+  'industries'=>'Отраслевые издания',
+  'job_hunting'=>'Поиск работы, карьера',
+  'literature_18'=>'Литература 18 века',
+  'literature_19'=>'Литература 19 века',
+  'literature_20'=>'Литература 20 века',
+  'love_contemporary'=>'Современные любовные романы',
+  'love_detective'=>'Остросюжетные любовные романы',
+  'love_erotica'=>'Эротическая литература',
+  'love_fantasy'=>'Любовное фэнтези',
+  'love_history'=>'Исторические любовные романы',
+  'love_sf'=>'Любовно-фантастические романы',
+  'love_short'=>'Короткие любовные романы',
+  'magician_book'=>'Книги про волшебников',
+  'management'=>'Управление, подбор персонала',
+  'marketing'=>'Маркетинг, PR, реклама',
+  'military_special'=>'Военное дело, спецслужбы',
+  'music_dancing'=>'Музыка, балет',
+  'narrative'=>'Повести',
+  'newspapers'=>'Газеты',
+  'nonfiction'=>'Документальная литература',
+  'nonf_biography'=>'Биографии и Мемуары',
+  'nonf_criticism'=>'Критика',
+  'nonf_publicism'=>'Публицистика',
+  'org_behavior'=>'Корпоративная культура',
+  'paper_work'=>'Делопроизводство',
+  'pedagogy_book'=>'Педагогика',
+  'periodic'=>'Журналы',
+  'personal_finance'=>'Личные финансы',
+  'poetry'=>'Поэзия',
+  'popadanec'=>'Попаданцы',
+  'popular_business'=>'О бизнесе популярно',
+  'prose_classic'=>'Классическая проза',
+  'prose_counter'=>'Контркультура',
+  'prose_history'=>'Историческая литература',
+  'prose_military'=>'Книги о войне',
+  'prose_rus_classic'=>'Русская классика',
+  'prose_su_classics'=>'Советская литература',
+  'psy_alassic'=>'Классики психологии',
+  'psy_childs'=>'Детская психология',
+  'psy_generic'=>'Общая психология',
+  'psy_personal'=>'Личностный рост',
+  'psy_sex_and_family'=>'Секс и семейная психология',
+  'psy_social'=>'Социальная психология',
+  'psy_theraphy'=>'Психотерапия и консультирование',
+  'real_estate'=>'Недвижимость',
+  'reference'=>'Справочная литература',
+  'ref_dict'=>'Словари',
+  'ref_encyc'=>'Энциклопедии',
+  'ref_guide'=>'Руководства',
+  'ref_ref'=>'Справочники',
+  'religion'=>'Религия',
+  'religion_esoterics'=>'Эзотерика',
+  'religion_rel'=>'Религиозные тексты',
+  'religion_self'=>'Самосовершенствование',
+  'russian_contemporary'=>'Современная русская литература',
+  'russian_fantasy'=>'Русское фэнтези',
+  'science'=>'Прочая образовательная литература',
+  'sci_biology'=>'Биология',
+  'sci_chem'=>'Химия',
+  'sci_culture'=>'Культурология',
+  'sci_history'=>'История',
+  'sci_juris'=>'Юриспруденция, право',
+  'sci_linguistic'=>'Языкознание',
+  'sci_math'=>'Математика',
+  'sci_medicine'=>'Медицина',
+  'sci_philosophy'=>'Философия',
+  'sci_phys'=>'Физика',
+  'sci_politics'=>'Политика, политология',
+  'sci_religion'=>'Религиоведение',
+  'sci_tech'=>'Техническая литература',
+  'sf'=>'Научная фантастика',
+  'sf_action'=>'Боевая фантастика',
+  'sf_cyberpunk'=>'Киберпанк',
+  'sf_detective'=>'Детективная фантастика',
+  'sf_heroic'=>'Героическая фантастика',
+  'sf_history'=>'Историческая фантастика',
+  'sf_horror'=>'Ужасы и Мистика',
+  'sf_humor'=>'Юмористическая фантастика',
+  'sf_social'=>'Социальная фантастика',
+  'sf_space'=>'Космическая фантастика',
+  'short_story'=>'Рассказы',
+  'sketch'=>'Очерки',
+  'small_business'=>'Малый бизнес',
+  'sociology_book'=>'Социология',
+  'stock'=>'Ценные бумаги, инвестиции',
+  'thriller'=>'Триллеры',
+  'upbringing_book'=>'Воспитание детей',
+  'vampire_book'=>'Книги про вампиров',
+  'visual_arts'=>'Изобразительное искусство, фотография',
+);
+
 sub FindFile {
   my $FileName = shift;
   my $Dirs = shift;
@@ -382,7 +569,7 @@ sub Reaper {
     my @P;
     my $Sec = SectionBody($X);
 
-    my $c=0;
+    $c=0;
     my $TitleOK = 0;
     push @{$Page->{'content'}},''; #пустышка-закрывашка (так легче обработать массив)
     foreach my $Item (@{$Page->{'content'}}) {
@@ -774,7 +961,6 @@ sub AssembleContent {
       next;
     }
 
-    my $Item = $ReverseManifest{$ItemID};
     if ($Item->{'type'} =~ /^application\/xhtml/) { # Видимо, текст
 
       my $ContentFile = $X->{'ContentDir'}.'/'.$Item->{'href'};
@@ -854,7 +1040,7 @@ sub AssembleContent {
         $node->parentNode()->removeChild($node);
       }
 
-      my $Content = $X->InNode($Body);
+      $Content = $X->InNode($Body);
       $Content =~ s/^\s*//;
       $Content =~ s/\s*$//;
       push @Pages, {
@@ -1046,4 +1232,243 @@ sub TransformH2Title {
   return $NewNode;
 }
 
+sub FB3Creator {
+	my $self = shift;
+	my $X = shift;
+  $X->Msg("Create FB3\n","w");
+
+  my $Structure = shift || $X->{'STRUCTURE'} || $X->Error("Structure is empty");
+  my $FB3Path = $X->{'DestinationDir'};
+
+  #compile required files
+  my $CoverSrc = $Structure->{'DESCRIPTION'}->{'TITLE-INFO'}->{'COVER'};
+
+  $X->Msg("FB3: Create /_rels/.rels\n","w");
+  my $FNrels="$FB3Path/_rels/.rels";
+  open FHrels, ">$FNrels" or $X->Error("$FNrels: $!");
+  print FHrels qq{<?xml version="1.0" encoding="UTF-8" standalone="yes"?>
+  <Relationships xmlns="http://schemas.openxmlformats.org/package/2006/relationships">}.
+  ( $CoverSrc ? qq{
+  <Relationship Id="rId0" Type="http://schemas.openxmlformats.org/package/2006/relationships/metadata/thumbnail" Target="$CoverSrc"/>} : '' ).qq{
+  <Relationship Id="rId1" Type="http://schemas.openxmlformats.org/package/2006/relationships/metadata/core-properties" Target="fb3/meta/core.xml"/>
+  <Relationship Id="rId2" Type="http://www.fictionbook.org/FictionBook3/relationships/Book" Target="fb3/description.xml"/>
+  </Relationships>};
+  close FHrels;
+
+  $X->Msg("FB3: Create [Content_Types].xml\n","w");
+  my $FNct="$FB3Path/[Content_Types].xml";
+  open FHct, ">$FNct" or $X->Error("$FNct: $!");
+  print FHct qq{<?xml version="1.0" encoding="UTF-8" standalone="yes"?>
+  <Types xmlns="http://schemas.openxmlformats.org/package/2006/content-types">
+   	<Default Extension="rels" ContentType="application/vnd.openxmlformats-package.relationships+xml" />
+   	<Default Extension="png" ContentType="image/png"/>
+   	<Default Extension="jpg" ContentType="image/jpeg"/>
+    <Default Extension="jpeg" ContentType="image/jpeg"/>
+   	<Default Extension="gif" ContentType="image/gif"/>
+   	<Default Extension="svg" ContentType="image/svg+xml"/>
+   	<Default Extension="xml" ContentType="application/xml"/>
+   	<Default Extension="css" ContentType="text/css"/>
+   	<Override PartName="/fb3/meta/core.xml" ContentType="application/vnd.openxmlformats-package.core-properties+xml"/>
+   	<Override PartName="/fb3/description.xml" ContentType="application/fb3-description+xml"/>
+   	<Override PartName="/fb3/body.xml" ContentType="application/fb3-body+xml"/>
+  </Types>};
+  close FHct;
+
+  $X->Msg("FB3: Create /fb3/_rels/description.xml.rels\n","w");
+  my $FNdrels="$FB3Path/fb3/_rels/description.xml.rels";
+  open FHdrels, ">$FNdrels" or $X->Error("$FNdrels: $!");
+  print FHdrels qq{<?xml version="1.0" encoding="UTF-8" standalone="yes"?>
+  <Relationships xmlns="http://schemas.openxmlformats.org/package/2006/relationships">
+   	<Relationship Id="rId0"
+    		Target="body.xml"
+    		Type="http://www.fictionbook.org/FictionBook3/relationships/body" />
+  </Relationships>};
+  close FHdrels;
+
+ # !! Стили пока не трогаем !!
+  #Скопируем стили
+#  foreach my $css (@{$Structure->{'CSS_LIST'}}) {
+#    my $SrcPath = $X->{'ContentDir'}."/".$css->{'src_path'};
+#    my $CssDestPath = $X->{'DestinationDir'}."/fb3/style";
+#    my $CssFile = $css->{'src_path'};
+#    $CssFile =~ s/.*\/([^\/]+)/$1/g;
+#    my $DstPath = $CssDestPath."/".$CssFile;
+#    $X->Msg("copy $SrcPath -> $DstPath\n","w");
+#    copy($SrcPath, $DstPath) or $X->Error($!);
+#    $css->{'new_path'} = "style/".$CssFile;
+#  }
+
+  #ДО ЭТОГО МОМЕНТА МЫ МОЖЕМ ИЗМЕНЯТЬ $Structure, ТО ЕСТЬ ВМЕШИВАТЬСЯ В КОНЕЧНЫЙ РЕЗУЛЬТАТ ОТРИСОВКИ FB3
+ # print Data::Dumper::Dumper($Structure);
+ # exit;
+  
+  my $GlobalID = $Structure->{'DESCRIPTION'}->{'DOCUMENT-INFO'}->{'ID'};
+  my $TitleInfo = $Structure->{'DESCRIPTION'}->{'TITLE-INFO'};
+  my $DocInfo = $Structure->{'DESCRIPTION'}->{'DOCUMENT-INFO'};
+  
+  #Пишем body
+  $X->Msg("FB3: Create /fb3/body.xml\n","w");
+  my $FNbody="$FB3Path/fb3/body.xml";
+  my $BodyAttr = {
+  	'xmlns'=>"http://www.fictionbook.org/FictionBook3/body",
+    'xmlns:xlink'=>"http://www.w3.org/1999/xlink",
+    'id'=>$GlobalID,
+  };
+
+  $X->_bs('Obj2DOM_body','PAGES => DOM');
+  my $Body = $X->Obj2DOM(
+              obj=>{
+                attributes=>{CP_compact=>1},
+                value=>$Structure->{'PAGES'}->{'value'}
+              },
+              root=>{name=>'fb3-body', attributes=>$BodyAttr}
+            );  
+  $X->_be('Obj2DOM_body');
+  
+  #финальное приведение section к валидному виду
+  foreach my $Section ($XC->findnodes( "/fb3-body/section/section", $Body), $XC->findnodes( "/fb3-body/section", $Body)) {
+    $Section = $X->Transform2Valid(node=>$Section);
+  }
+
+  #финальное приведение table к валидному виду
+  foreach my $Table ($XC->findnodes( "/fb3-body//table", $Body)) {
+    $Table = $X->TransformTable2Valid(node=>$Table);
+  }
+
+  open FHbody, ">$FNbody" or $X->Error("$FNbody: $!");
+  print FHbody $Body->toString(1);
+  close FHbody;
+
+  #Пишем мету
+  #Превращаем перл-структуру в DOM
+  delete $Structure->{'PAGES'};
+  $X->_bs('Obj2DOM_meta','META => DOM');
+  my $Doc = $X->Obj2DOM(obj=>$Structure, like_parent=>0, compact=>0 ); 
+  $X->_be('Obj2DOM_meta');
+
+  #Пишем rels
+  $X->Msg("FB3: Create /fb3/_rels/body.xml.rels\n","w");
+  my $FNbodyrels="$FB3Path/fb3/_rels/body.xml.rels";
+  open FHbodyrels, ">$FNbodyrels" or $X->Error("$FNbodyrels: $!");
+  print FHbodyrels qq{<?xml version="1.0" encoding="UTF-8"?>
+<Relationships xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns="http://schemas.openxmlformats.org/package/2006/relationships" xmlns:xlink="http://www.w3.org/1999/xlink">
+};
+  foreach (@{$Structure->{'IMG_LIST'}}) {
+    print FHbodyrels qq{  <Relationship Id="$_->{'id'}" Type="http://www.fictionbook.org/FictionBook3/relationships/image" Target="$_->{'new_path'}"/>
+};
+  }
+  print FHbodyrels qq{</Relationships>};
+  close FHbodyrels;
+  
+  #Пишем core
+  $X->Msg("FB3: Create /fb3/meta/core.xml\n","w");
+  my $FNcore="$FB3Path/fb3/meta/core.xml";
+  open FHcore, ">$FNcore" or $X->Error("$FNcore: $!");
+  print FHcore qq{<?xml version="1.0" encoding="UTF-8"?>
+  <cp:coreProperties xmlns:cp="http://schemas.openxmlformats.org/package/2006/metadata/core-properties" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xlink="http://www.w3.org/1999/xlink" xmlns="http://www.fictionbook.org/FictionBook3/description" xmlns:dc="http://purl.org/dc/elements/1.1/" xmlns:dcterms="http://purl.org/dc/terms/" xmlns:dcmitype="http://purl.org/dc/dcmitype/">
+    <dc:title>}.$TitleInfo->{'BOOK-TITLE'}.qq{</dc:title>
+    <dc:subject>}.$TitleInfo->{'ANNOTATION'}.qq{</dc:subject>
+    <dc:creator>};
+  
+  my $c=1;         
+  foreach (@{$TitleInfo->{'AUTHORS'}}) {
+    print FHcore $_->{'first-name'}." ".$_->{'last-name'};
+    print FHcore ", " if $c < scalar @{$TitleInfo->{'AUTHORS'}};
+    $c++;
+  }
+  
+  print FHcore qq{</dc:creator>
+    <dc:description>}.$TitleInfo->{'ANNOTATION'}.qq{</dc:description>
+    <cp:keywords>XML, FictionBook, eBook, OPC</cp:keywords>
+    <cp:revision>1.00</cp:revision>
+    }.
+  ($DocInfo->{'DATE'}->{'attributes'}->{'value'}?qq{<dcterms:created xsi:type="dcterms:W3CDTF">}.$DocInfo->{'DATE'}->{'attributes'}->{'value'}.qq{</dcterms:created>
+    }:'')
+  .qq{<cp:contentStatus>Draft</cp:contentStatus>
+    <cp:category>};
+  
+  $c=1;         
+  foreach (@{$TitleInfo->{'GENRES'}}) {
+    print FHcore $_;
+    print FHcore ", " if $c < scalar @{$TitleInfo->{'GENRES'}};
+    $c++;
+  }
+
+  print FHcore qq{</cp:category>
+  </cp:coreProperties>};
+  close FHcore;
+  #Пишем description
+  $X->Msg("FB3: Create /fb3/description.xml\n","w");
+  my $FNdesc="$FB3Path/fb3/description.xml";
+  open FHdesc, ">$FNdesc" or $X->Error("$FNdesc: $!");
+  print FHdesc qq{<?xml version="1.0" encoding="UTF-8"?>
+<fb3-description xmlns="http://www.fictionbook.org/FictionBook3/description" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xlink="http://www.w3.org/1999/xlink" id="}.$GlobalID.qq{" version="1.0">
+  };
+
+  print FHdesc qq{<title>
+    <main>}.$TitleInfo->{'BOOK-TITLE'}.qq{</main>
+  </title>
+  };
+    
+  print FHdesc qq{<fb3-relations>
+    };
+
+  foreach (@{$TitleInfo->{'AUTHORS'}}) {
+
+    my $First = $_->{'first-name'};
+    my $Middle = $_->{'middle-name'};
+    my $Last = $_->{'last-name'}||"Unknown";
+
+    print FHdesc qq{<subject link="author" id="}.($_->{'id'}||'00000000-0000-0000-0000-000000000000').qq{">
+      <title>
+        <main>
+          }.
+      ($First?$First.' '.$Last:$Last).qq{
+        </main>
+      </title>}
+      .($First?qq{
+      <first-name>}.$First.'</first-name>':'')
+      .($Middle?qq{
+      <middle-name>}.$Middle.qq{</middle-name>}:'')
+      .qq{
+      <last-name>}.$Last.qq{</last-name>
+    </subject>
+  };
+  }
+
+  print FHdesc qq{</fb3-relations>
+  };
+
+  
+  print FHdesc qq{<fb3-classification>
+    };
+  foreach (@{$TitleInfo->{'GENRES'}}) {
+    print FHdesc qq{<subject>}.(exists $GenreTranslate{$_}?$GenreTranslate{$_}:$_).qq{</subject>
+    };
+  }
+  print FHdesc qq{</fb3-classification>
+  };
+  
+
+  print FHdesc qq{<lang>}.$DocInfo->{'LANGUAGE'}.qq{</lang>
+  <written>
+    <lang>}.$DocInfo->{'LANGUAGE'}.qq{</lang>
+  </written>
+  };
+
+  my $DCDate = $DocInfo->{'DATE'}->{'attributes'}->{'value'}."T".$DocInfo->{'TIME'}->{'attributes'}->{'value'};
+  print FHdesc qq{<document-info created="}.$DCDate.qq{" updated="}.$DCDate.qq{"/>
+};
+
+  print FHdesc qq{  <annotation>
+    <p>}.$TitleInfo->{'ANNOTATION'}.qq{</p>
+  </annotation>
+} if $TitleInfo->{'ANNOTATION'};
+
+  print FHdesc qq{</fb3-description>};
+  close FHdesc;
+ 
+  return $FB3Path;
+}
+
 1;
